# start up the main Qt application and load plugins

from PySide6.QtCore import QCoreApplication, QObject, Signal, QTimer
from PySide6.QtWidgets import QApplication, QToolBar, QWidget, QMainWindow, QPlainTextEdit
import logging
from logging import LogRecord
import sys
from pyqtgraph.dockarea import Dock, DockArea

from solarcarsim.satellaview.ui import SatellaUI


class LogHandler(logging.Handler):
    class Carrier(QObject):
        # We need this because both QObject and logging.Handler need an `emit` method
        # and they collide.
        appendplaintext = Signal(str)

    def __init__(self, parent) -> None:
        super().__init__()
        self.widget = QPlainTextEdit(parent=parent)
        self.carrier = self.Carrier(parent=parent)
        self.widget.setReadOnly(True)
        self.carrier.appendplaintext.connect(self.widget.appendPlainText) # type: ignore
        self.dock = Dock("Logger", widget=self.widget)

    def emit(self, record: LogRecord) -> None:
        msg = self.format(record)
        self.carrier.appendplaintext.emit(msg) # type: ignore


class LogViewer(QPlainTextEdit):
    def __init__(self, parent=None):
        super().__init__(parent)


class App():
    """ Core application. Sets up logger, main window (toolbar, etc),
    and loads plugins."""
    @staticmethod
    def init_core_app():
        # sets the name of the application etc
        QCoreApplication.setApplicationName("SolarCarSim")

    def __init__(self) -> None:
        self.main_window = main = QMainWindow()
        main.setWindowTitle("SolarCarSim")
        self.dockarea = DockArea(main)

        self.loghandler = LogHandler(main)
        self.plugins = {}

        self.plugins['satellaview'] = SatellaUI(main)
        logging.getLogger().addHandler(self.loghandler)
        
        main.setCentralWidget(self.dockarea)
        self.dockarea.addDock(self.loghandler.dock)
        self.dockarea.addDock(self.plugins['satellaview'].dock)
        self.file_menu = main.menuBar().addMenu("File")
        self.settings_menu = main.menuBar().addMenu("Settings")



    def run(self):
        return self.main_window.show()
    

if __name__ == "__main__":
    app = QApplication()
    myapp = App()
    myapp.run()
    QTimer.singleShot(1000, lambda: logging.warning("logging test"))
    QTimer.singleShot(2000, lambda: logging.warning("logging tes2332t"))
    sys.exit(app.exec())